#  ---------------------------------------------------------------------
#   Database implementation of ODL-Rate (v1.0) - data provider
#    v1.0
#    March 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


package ODL::DBRate::DBRateDP;


use POSIX;

use Pure::EZXML;

use ODL::Box::BoxDP;
use vars ('@ISA');
@ISA = ("ODL::Box::BoxDP");


# constructor
sub new
{
   my ($classname, $configfile) = @_;
   my $self = $classname->SUPER::new ($configfile);
   bless $self, $classname;
   return $self;
}


# destructor
sub dispose
{
   my ($self) = @_;
   $self->SUPER::dispose ();
}


# component-specific identification
sub Archive_Identify
{
   my ($self) = @_;
   
   my $id = $self->SUPER::Archive_Identify ();
      
   if (! exists $id->{'description'})
   {
      $id->{'description'} = [];
   }
   my $desc = {
      'odl-description' => [[ 
         {
            'xmlns' => $self->{xoai_xmlnsprefix}.'/odl-description',
            'xsi:schemaLocation' => 
                       $self->{xoai_xmlnsprefix}.'/odl-description '.
                       $self->{xoai_xmlnsprefix}.'/odl-description.xsd'
         },
         {
            'protocol' => 'ODL-Rate',
            'version'  => '1.0',
            'mdorder'  => [ qw ( protocol version ) ]
         } 
      ]]
   };
   push (@{$id->{'description'}}, $desc);
   
   $id;
}


# retrieve records from the source archive as required
sub Archive_FormatRecord
{
   my ($self, $hashref, $metadataFormat) = @_;
   
   if (defined $hashref->{$metadataFormat})
   {
      my $rowref = $hashref->{$metadataFormat};
      $self->FormatRecord ($rowref->{'identifier'},
                           $self->toISO8601 ($rowref->{'datestamp'}, 'second'),
                           $rowref->{'metadata'},
                           $rowref->{'about'},
                           $rowref->{'status'}
                          );
   }
   elsif ((keys %$hashref) == 0)
   {
      return '';
   }
   else
   {
      my $firstkey = (keys %$hashref)[0];
      my $rowref = $hashref->{$firstkey};
      $self->FormatRecord ($rowref->{'identifier'},
                           $self->toISO8601 ($rowref->{'datestamp'}, 'second'),
                           '',
                           '',
                           $rowref->{'status'}
                          );
   }
}


# get full list of mdps or list for specific identifier
sub Archive_ListMetadataFormats
{
   my ($self, $identifier) = @_;
   
   my ($metadataNamespace, $metadataSchema) = $self->SUPER::Archive_ListMetadataFormats ($identifier);

   ( $metadataNamespace, $metadataSchema );
}


# pass request through to source
sub Archive_GetRecord
{
   my ($self, $identifier, $metadataPrefix) = @_;
   
   my %records = ();
   
   if ($metadataPrefix eq 'odl_rating')
   {
      my $sth = $self->{'dbh'}->prepare ("select identifier,status,datestamp,metadataPrefix,metadata,about from $self->{'table'} where identifier=\'$identifier\'");
      $sth->execute;

      my $rowref;
      while ($rowref = $sth->fetchrow_hashref)
      {
         $records{$rowref->{'metadataPrefix'}} = {
            identifier => $rowref->{'identifier'},
            datestamp  => $rowref->{'datestamp'},
            status     => $rowref->{'status'},
            metadata   => $rowref->{'metadata'},
            about      => $rowref->{'about'}
         };
      }

      $sth->finish;
   }
   elsif ($metadataPrefix eq 'rating_average')
   {
      my $sum = 0;
      my $count = 0;
      my $ratings = {};
      my $datestamps = {};
      
      my $sth = $self->{'dbh'}->prepare ("select datestamp,metadata from $self->{'table'},$self->{'table'}set where $self->{'table'}.identifier=$self->{'table'}set.identifier and $self->{'table'}set.oaiset=\'$identifier\'");
      $sth->execute;
      
      my $rowref;
      while ($rowref = $sth->fetchrow_hashref)
      {
         my $parser = new Pure::EZXML;
         my $document = $parser->parse ($rowref->{'metadata'})->getDocumentElement;
         
         my $ratingtag = $document->getElementsByTagName ('rating', 0);
         
         if ($ratingtag->getLength == 1)
         {
            my $rating = $ratingtag->item(0)->getChildNodes->toString;
            my $subjecttag = $document->getElementsByTagName ('subject', 0);
            if ($subjecttag->getLength == 1)
            {
               my $subject = $subjecttag->item(0)->getChildNodes->toString;
               my $datestamp = $rowref->{'datestamp'};
               if (((defined $datestamps->{$subject}) && ($datestamp > $datestamps->{$subject})) ||
                   (! defined $datestamps->{$subject}))
               {
                  $datestamps->{$subject} = $datestamp;
                  $ratings->{$subject} = $rating;
               }
            }
         }
      }
      
      $sth->finish;
      
      foreach my $subject (keys %{$ratings})
      {
         $sum += $ratings->{$subject};
         $count++;
      }
      
      my $average = 0;
      if ($count > 0)
      {
         $average = $sum / $count;
      }
      
      $records{'rating_average'} = {
         identifier => $identifier,
         datestamp  => time,
         metadata   => "<odl_rating_average><average>$average</average><number>$count</number></odl_rating_average>",
      };
   }

   \%records;
}


# return list of records/identifiers
sub Archive_ListRecords
{
   my ($self, $set, $from, $until, $metadataPrefix, $resumptionToken) = @_;

   $self->SUPER::Archive_ListRecords ($set, $from, $until, $metadataPrefix, $resumptionToken);
}


# insert a new record into the archive
sub Archive_PutRecord
{
   my ($self, $identifier, $datestamp, $metadata, $metadataPrefix,
       $metadataNamespace, $metadataSchema, $about, $status, @sets) = @_;
       
   my $sth;

   # get unique identifier if necessary
   if ($identifier eq '')
   {
      $sth = $self->{'dbh'}->prepare ("lock tables $self->{'table'}admin write");
      $sth->execute;
      $sth->finish;
      $sth = $self->{'dbh'}->prepare ("select value from $self->{'table'}admin where variable=\'counter\'");
      $sth->execute;
      if (my $rowref = $sth->fetchrow_hashref)
      {
         $identifier = "oai:$self->{'archiveId'}:$rowref->{'value'}";
      }
      $sth->finish;
      $sth = $self->{'dbh'}->prepare ("update $self->{'table'}admin set value=value+1 where variable=\'counter\'");
      $sth->execute;
      $sth->finish;
      $sth = $self->{'dbh'}->prepare ("unlock tables");
      $sth->execute;
      $sth->finish;
   }
   
   if ($status eq 'deleted')
   {
      # delete everything
      $sth = $self->{'dbh'}->prepare ("update $self->{'table'} set status=\'deleted\' where identifier=\'$identifier\'");
      $sth->execute;
      $sth->finish;
   }
   else
   {
      # add basic metadata
      $self->{'dbh'}->do ("delete from $self->{'table'} where identifier=\'$identifier\' and metadataPrefix=\'$metadataPrefix\'");
      $self->{'dbh'}->do ("insert into $self->{'table'} set identifier=\'$identifier\', ".
                          "datestamp=\'$datestamp\', ".
                          "metadata=".$self->{'dbh'}->quote ($metadata).', '.
                          "metadataPrefix=\'$metadataPrefix\', ".
                          "about=".$self->{'dbh'}->quote ($about).', '.
                          "status=\'$status\'");

      # get name of the object for the sets
      @sets = ();
      my $parser = new Pure::EZXML;
      my $document = $parser->parse ($metadata)->getDocumentElement;
      my $object_tag = $document->getElementsByTagName ('object', 0);
      if ($object_tag->getLength == 1)
      {
         my $object = $object_tag->item(0)->getChildNodes->toString;
         $sets[0] = $object;
      }

      # add sets for resources based on transaction result
      $self->{'dbh'}->do ("delete from $self->{'table'}set where identifier=\'$identifier\'");
      foreach my $aset (@sets)
      {
         $self->{'dbh'}->do ("insert into $self->{'table'}set set identifier=\'$identifier\', oaiset=\'$aset\'");
      }

      # add metadata namespace/schema
      $self->{'dbh'}->do ("delete from $self->{'table'}md where metadataPrefix=\'$metadataPrefix\' and metadataNamespace=\'$metadataNamespace\' and metadataSchema=\'$metadataSchema\'");
      $self->{'dbh'}->do ("insert into $self->{'table'}md set metadataPrefix=\'$metadataPrefix\', metadataNamespace=\'$metadataNamespace\', metadataSchema=\'$metadataSchema\'");
   }

   $identifier;
}


1;

